import java.lang.reflect.*;

/**
 * CPSC 501
 * Inspector starter class
 *
 * @author Jonathan Hudson
 */
public class Inspector {
    public void inspect(Object obj, boolean recursive) {
        Class<?> c = obj.getClass();

        // Handle array objects
        if (c.isArray()) {
            inspectArray(obj, recursive, 0);
        } else {
            inspectClass(c, obj, recursive, 0);
        }
    }
    private void inspectClass(Class<?> c, Object obj, boolean recursive, int depth) {
        if(c == null) return;

        // Get the proper indent amount
        String indent = getIndent(depth);

        // Name of declaring class
        System.out.println(indent + "Class: " + c.getName());

        // Name of immediate super-class
        Class<?> sClass = c.getSuperclass();
        if (sClass != null) {
            System.out.println(indent + "Superclass: " + sClass.getName());
            // Explore superclasses recursively right away
            inspectClass(sClass, null, recursive, depth + 1);
        }

        // Name of each interface
        Class<?>[] inters = c.getInterfaces();
        if (inters.length > 0) {
            System.out.println(indent + "Interfaces:");
            for (Class<?> inter : inters) {
                System.out.println(indent + " " + inter.getName());
                // Explore interfaces recursively right away
                inspectClass(inter, null, recursive, depth + 1);
            }
        } else {
            System.out.println(indent + "Interfaces: NONE");
        }

        // Constructors
        inspectConstructors(c, depth);

        // Methods
        inspectMethods(c, depth);

        // Fields
        inspectFields(c, obj, recursive, depth);
    }

    private void inspectConstructors(Class<?> c, int depth) {
        //Get proper indentation
        String indent = getIndent(depth);

        Constructor<?>[] constructors = c.getDeclaredConstructors();

        if(constructors.length == 0) {
            System.out.println(indent + "Constructors: NONE");
        }
        else{
            System.out.println(indent + "Constructors: ");
        }

        for (Constructor<?> constructor : constructors) {
            // Name of constructor
            System.out.println(indent + " Constructor: " + constructor.getName());

            // Exceptions thrown
            Class<?>[] eTypes = constructor.getExceptionTypes();
            if (eTypes.length == 0) {
                System.out.println(indent + "  Exceptions thrown: NONE");
            } else {
                System.out.println(indent + "  Exceptions thrown:");
                for (Class<?> e : eTypes) {
                    System.out.println(indent + "       " + e.getName());
                }
            }
            // Parameter types
            Class<?>[] pTypes = constructor.getParameterTypes();

            if (pTypes.length == 0) {
                System.out.println(indent + "  Parameter types: NONE");
            } else {
                System.out.println(indent + "  Parameter types:");
                for (Class<?> p : pTypes) {
                    System.out.println(indent + "       " + p.getName());
                }
            }

            // Modifiers
            System.out.println(indent + "  Modifiers: " + Modifier.toString(constructor.getModifiers()));

            System.out.println(); // Empty line for readability
        }
    }

    private void inspectMethods(Class<?> c, int depth) {
        String indent = getIndent(depth);
        Method[] methods = c.getDeclaredMethods();

        if (methods.length == 0) {
            System.out.println(indent + "Methods: NONE");
        }
        else{
            System.out.println(indent + "Methods:");
        }

        for (Method method : methods) {
            // Name
            System.out.println(indent + " Method: " + method.getName());

            // Exceptions thrown
            Class<?>[] exceptionTypes = method.getExceptionTypes();
            if (exceptionTypes.length == 0) {
                System.out.println(indent + "  Exceptions thrown: NONE");
            }
            else{
                System.out.println(indent + "  Exceptions thrown:");
                for (Class<?> e : exceptionTypes) {
                    System.out.println(indent + "       " + e.getName());
                }
            }

            // Parameter types
            Class<?>[] paramTypes = method.getParameterTypes();
            if (paramTypes.length == 0) {
                System.out.println(indent + "  Parameter types: NONE");
            } else {
                System.out.println(indent + "  Parameter types:");
                for (Class<?> param : paramTypes) {
                    System.out.println(indent + "       " + param.getName());
                }
            }

            // Return-type
            System.out.println(indent + "  Return type: " + method.getReturnType().getName());

            // Modifiers
            System.out.println(indent + "  Modifiers: " + Modifier.toString(method.getModifiers()));

            System.out.println(); // Empty line for readability
        }
    }

    private void inspectFields(Class<?> c, Object obj, boolean recursive, int depth) {
        String indent = getIndent(depth);
        Field[] fields = c.getDeclaredFields();

        if (fields.length == 0) {
            System.out.println(indent + "Fields: NONE");
        }
        else{
            System.out.println(indent + "Fields: ");
        }

        for (Field field : fields) {
            try {
                field.setAccessible(true);

                // Name
                System.out.println(indent + " Field: " + field.getName());

                // Type
                System.out.println(indent + "  Type: " + field.getType().getName());

                // Modifiers
                System.out.println(indent + "  Modifiers: " + Modifier.toString(field.getModifiers()));

                // Current value
                if (obj != null) {
                    try {
                        Object value = field.get(obj);
                        if (value == null) {
                            System.out.println(indent + "  Current Value: null");
                        } else {
                            System.out.println(indent + "  Current Value: " + value);

                            if (recursive && !field.getType().isPrimitive()) {
                                inspectClass(value.getClass(), value, recursive, depth + 1);
                            }
                        }
                    } catch (IllegalAccessException e) {
                        System.out.println(indent + "  Current Value: <Could not access>");
                    }
                } else {
                    System.out.println(indent + "  Current Value: <No instance provided>");
                }
            } catch (Exception e) {
                System.out.println(indent + " Error inspecting field: " + e.getMessage());
            }

            System.out.println(); // Empty line for readability
        }
    }

    private void inspectArray(Object array, boolean recursive, int depth) {
        if (array == null) {
            return;
        }

        String indent = getIndent(depth);
        Class<?> arrayClass = array.getClass();

        System.out.println(indent + "Array: " + arrayClass.getName());

        // Component Type
        System.out.println(indent + "Type: " + arrayClass.getTypeName());

        // Length
        int length = Array.getLength(array);
        System.out.println(indent + "Length: " + length);

        // Values
        System.out.println(indent + "Entry Values:");
        for (int i = 0; i < length; i++) {
            Object value = Array.get(array, i);
            if (value == null) {
                System.out.println(indent + " [" + i + "]: null");
            } else {
                System.out.println(indent + " [" + i + "]: " + value);
            }

            System.out.println(); // Empty line for readability
        }
    }
    // Helper function to get proper indentation based on depth
    private String getIndent(int depth) {
        StringBuilder indentBuilder = new StringBuilder();
        for (int i = 0; i < depth; i++) {
            indentBuilder.append("\t");
        }
        return indentBuilder.toString();
    }
}