import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;

import static org.junit.jupiter.api.Assertions.*;

import java.io.ByteArrayOutputStream;
import java.io.PrintStream;

public class InspectorTest {
    private Inspector inspector;
    private ByteArrayOutputStream outContent;
    private PrintStream originalOut;

    // SAMPLE CLASSES AND INTERFACES
    class TestClass {
        private int x = 10;
        public String name = "Test";

        public void doSomething() {}
    }

    interface TestInterface {
        void interfaceMethod();
    }

    class TestChildClass extends TestClass implements TestInterface {
        private double value = 100;

        public TestChildClass() {}

        public TestChildClass(double val) {
            this.value = val;
        }

        @Override
        public void interfaceMethod() {}

        public double getValue() {
            return value;
        }
    }

    class ClassWithNestedObjects {
        private TestClass simpleObj = new TestClass();
        private TestChildClass childObj = new TestChildClass(7.5);
    }

    @BeforeEach
    public void setUp() {
        inspector = new Inspector();
        // Setup output capture
        originalOut = System.out;
        outContent = new ByteArrayOutputStream();
        System.setOut(new PrintStream(outContent));
    }

    public void tearDown() {
        System.setOut(originalOut);
    }

    private String getOutput() {
        return outContent.toString();
    }

    @Test
    public void testInspectSimpleClass() {
        TestClass obj = new TestClass();
        inspector.inspect(obj, false);

        String output = getOutput();

        assertTrue(output.contains("Class: " + TestClass.class.getName()));
        assertTrue(output.contains("Field: x"));
        assertTrue(output.contains("Field: name"));
        assertTrue(output.contains("Type: int"));
        assertTrue(output.contains("Type: java.lang.String"));
        assertTrue(output.contains("Current Value: Test"));
    }

    @Test
    public void testConstructorInspection() {
        TestChildClass child = new TestChildClass();
        inspector.inspect(child, false);

        String output = getOutput();

        assertTrue(output.contains("Constructors:"));
        assertTrue(output.contains("Constructor: " + TestChildClass.class.getName()));
        assertTrue(output.contains("Parameter types:"));
        assertTrue(output.contains("double"));
    }

    @Test
    public void testMethodInspection() {
        TestChildClass child = new TestChildClass();
        inspector.inspect(child, false);

        String output = getOutput();

        assertTrue(output.contains("Methods:"));
        assertTrue(output.contains("Method: getValue"));
        assertTrue(output.contains("Method: interfaceMethod"));
        assertTrue(output.contains("Return type: double"));
        assertTrue(output.contains("Return type: void"));
    }

    @Test
    public void testArrayInspection() {
        int[] numbers = {1, 2, 3, 4, 5};
        inspector.inspect(numbers, false);

        String output = getOutput();

        assertTrue(output.contains("Array: [I"));
        assertTrue(output.contains("Length: 5"));
    }

    @Test
    public void testRecursiveInspection() {
        ClassWithNestedObjects nestedObj = new ClassWithNestedObjects();
        inspector.inspect(nestedObj, true);

        String output = getOutput();

        assertTrue(output.contains("Class: " + ClassWithNestedObjects.class.getName()));
        assertTrue(output.contains("Class: " + TestClass.class.getName()));
        assertTrue(output.contains("Class: " + TestChildClass.class.getName()));
        assertTrue(output.contains("Field: x"));
        assertTrue(output.contains("Field: value"));
    }

    @Test
    public void testNonRecursiveInspection() {
        ClassWithNestedObjects nestedObj = new ClassWithNestedObjects();
        inspector.inspect(nestedObj, false);

        String output = getOutput();

        assertTrue(output.contains("Class: " + ClassWithNestedObjects.class.getName()));
        assertFalse(output.contains("Class: " + TestClass.class.getName() + "\n"));
    }
}